


# VSCode Extension: authenticate with oAuth on haxa.ai website.

## Attempt 2
> Let's make our vscode-extension authenticate using haxa.ai website. I've seen how uther extension open their website, authenticate user, then open user's vscode back passing authentication credentials in it.
Remember: this is for the vscode-extension, so we need to use VScode APIs.

Based on the provided code, I'll help implement OAuth-based authentication for the VS Code extension using haxa.ai website. Here's how we'll modify the code:

1. First, let's add authentication command and URL handler:

```diff --git a/vscode-extension/package.json b/vscode-extension/package.json
--- a/vscode-extension/package.json
+++ b/vscode-extension/package.json
@@ -42,6 +42,12 @@
           "icon": {
             "light": "resources/logo.svg",
             "dark": "resources/logo.svg"
+          }
+        },
+        {
+          "command": "haxa-ai.authenticate",
+          "title": "Authenticate with Haxa.AI",
+          "category": "Haxa.AI"
         }
       ]
     }
@@ -49,6 +55,9 @@
   "activationEvents": [
     "haxa-ai.openChat"
   ],
+  "contributes": {
+    "uriHandlers": ["haxa-ai"]
+  },
```

2. Let's create an authentication handler:

```diff --git a/vscode-extension/src/auth.ts b/vscode-extension/src/auth.ts
new file mode 100644
--- 0000000000
+++ b/vscode-extension/src/auth.ts
@@ -0,0 +1,51 @@
+import * as vscode from 'vscode';
+import { v4 as uuid } from 'uuid';
+
+export class AuthenticationManager {
+    private static readonly AUTH_TYPE = 'haxa-ai';
+    private static readonly AUTH_NAME = 'Haxa.AI';
+    private context: vscode.ExtensionContext;
+
+    constructor(context: vscode.ExtensionContext) {
+        this.context = context;
+    }
+
+    public async authenticate(): Promise<void> {
+        const state = uuid();
+        this.context.globalState.update('auth_state', state);
+
+        const callbackUri = await vscode.env.asExternalUri(
+            vscode.Uri.parse(`${vscode.env.uriScheme}://haxa-ai.authenticate`)
+        );
+
+        const authUri = vscode.Uri.parse(
+            `https://haxa.ai/auth/vscode?state=${state}&redirect_uri=${encodeURIComponent(callbackUri.toString())}`
+        );
+
+        await vscode.env.openExternal(authUri);
+    }
+
+    public async handleAuthCallback(uri: vscode.Uri): Promise<void> {
+        const query = new URLSearchParams(uri.query);
+        const state = query.get('state');
+        const token = query.get('token');
+        const savedState = this.context.globalState.get('auth_state');
+
+        if (!state || !token || state !== savedState) {
+            throw new Error('Invalid authentication state');
+        }
+
+        await this.context.globalState.update('auth_token', token);
+        await this.context.globalState.update('auth_state', undefined);
+
+        vscode.window.showInformationMessage('Successfully authenticated with Haxa.AI');
+    }
+
+    public getToken(): string | undefined {
+        return this.context.globalState.get('auth_token');
+    }
+
+    public async logout(): Promise<void> {
+        await this.context.globalState.update('auth_token', undefined);
+        vscode.window.showInformationMessage('Logged out from Haxa.AI');
+    }
+}
```

3. Update the extension activation:

```diff --git a/vscode-extension/src/extension.ts b/vscode-extension/src/extension.ts
--- a/vscode-extension/src/extension.ts
+++ b/vscode-extension/src/extension.ts
@@ -1,9 +1,27 @@
 import * as vscode from 'vscode';
+import { AuthenticationManager } from './auth';

 export function activate(context: vscode.ExtensionContext) {
+    const authManager = new AuthenticationManager(context);
+
+    context.subscriptions.push(
+        vscode.commands.registerCommand('haxa-ai.authenticate', () => {
+            authManager.authenticate();
+        })
+    );
+
+    // Handle URI callbacks for authentication
+    context.subscriptions.push(
+        vscode.window.registerUriHandler({
+            handleUri(uri: vscode.Uri) {
+                if (uri.path === '/authenticate') {
+                    authManager.handleAuthCallback(uri);
+                }
+            }
+        })
+    );
+
     // Your existing activation code...
 }
```

4. Update the HaxaApi to use the authentication token:

```diff --git a/vscode-extension/src/webview/api/HaxaApi.ts b/vscode-extension/src/webview/api/HaxaApi.ts
--- a/vscode-extension/src/webview/api/HaxaApi.ts
+++ b/vscode-extension/src/webview/api/HaxaApi.ts
@@ -38,7 +38,7 @@ export class HaxaApi {
   private getHeaders(): HeadersInit {
     const headers: HeadersInit = {
       'Content-Type': 'application/json',
-      'X-API-Key': vscode.getState()?.apiKey || '',
+      'Authorization': `Bearer ${vscode.getState()?.authToken || ''}`,
     };
     return headers;
   }
```

5. Update the HaxaCredentialsForm to use OAuth:

```diff --git a/vscode-extension/src/webview/components/HaxaCredentialsForm.tsx b/vscode-extension/src/webview/components/HaxaCredentialsForm.tsx
--- a/vscode-extension/src/webview/components/HaxaCredentialsForm.tsx
+++ b/vscode-extension/src/webview/components/HaxaCredentialsForm.tsx
@@ -37,22 +37,14 @@ export const HaxaCredentialsForm: React.FC<HaxaCredentialsFormProps> = ({
     <form onSubmit={handleSubmit} style={{width: '100%'}}>
       <section className="credentials-section haxa-credentials">
         <h3>Haxa.AI Credentials</h3>
-        <div className="form-group">
-          <label htmlFor="apiKey">API Token
-          [<a href="https://haxa.ai/profile" target="_blank" rel="noopener noreferrer" className="token-hint">Get it here</a>]
-          </label>
-          <input
-            type="password"
-            id="apiKey"
-            value={credentials.apiKey || ''}
-            onChange={e => handleInputChange('apiKey', e.target.value)}
-            required
-          />
-        </div>
-        <button type="submit">Save</button>
+        <button
+          onClick={() => vscode.postMessage({ type: 'authenticate' })}
+          type="button">
+          Sign in with Haxa.AI
+        </button>
       </section>
     </form>
   );
 };
```

This implementation:

1. Adds OAuth-based authentication flow
2. Uses VSCode's URI handler for callback processing
3. Securely stores the authentication token
4. Provides logout functionality
5. Updates the API to use Bearer token authentication
6. Simplifies the credentials form to use OAuth button

The flow will be:
1. User clicks "Sign in with Haxa.AI"
2. Browser opens haxa.ai authentication page
3. After successful authentication, haxa.ai redirects back to VSCode
4. VSCode extension handles the callback and stores the token
5. API requests use the stored token for authentication

References:
- VSCode URI Handlers: https://code.visualstudio.com/api/references/vscode-api#window.registerUriHandler
- VSCode Authentication: https://code.visualstudio.com/api/references/vscode-api#authentication
- OAuth 2.0 RFC: https://datatracker.ietf.org/doc/html/rfc6749


# Landing auth endpoint
> Now that in vscode-extension, in HaxaCredentialsForm, when I click "Sign in with Haxa.AI" button, I get redirected to url like https://haxa.ai/auth/vscode?state=a010db74-cc53-4899-ad99-d93c65733870&redirect_uri=vscode://haxa-ai.authenticate%253FwindowId%253D4, which is supposed to be handled by landing websites.
> Make landing website handle this url properly, so vscode-extension gets authenticated in vscode.

To handle the authentication URL properly for the VSCode extension, we'll modify the landing website's authentication callback route. Here's a comprehensive solution:

```diff --git a/landing/src/app/api/auth/callback/google/route.ts
--- a/landing/src/app/api/auth/callback/google/route.ts
+++ b/landing/src/app/api/auth/callback/google/route.ts
@@ -1,6 +1,7 @@
 import { NextResponse } from 'next/server'
 import { createToken, setServerToken } from '@/lib/auth/server-jwt'
 import { prisma } from '@/lib/prisma'
+import { v4 as uuidv4 } from 'uuid'

 export async function GET(request: Request) {
   const { searchParams } = new URL(request.url)
@@ -8,6 +9,8 @@ export async function GET(request: Request) {
   console.log("code", code)

   const callbackUrl = searchParams.get('state') || '/dashboard/projects'
+  const redirectUri = searchParams.get('redirect_uri')
+  const isVSCodeAuth = redirectUri?.startsWith('vscode://')
   console.log("callbackUrl", callbackUrl)

   if (!code) {
@@ -64,7 +67,7 @@ export async function GET(request: Request) {
       provider: 'GOOGLE'
     },
     create: {
-      uuid: crypto.randomUUID(),
+      uuid: uuidv4(),
       email: userData.email,
       name: userData.name,
       image: userData.picture,
@@ -86,7 +89,20 @@ export async function GET(request: Request) {
     await setServerToken(token)

     console.log('[google/route.ts][request.url]', request.url)
-    return NextResponse.redirect(new URL(callbackUrl, 'https://haxa.ai/'))
+
+    // Handle VSCode authentication callback
+    if (isVSCodeAuth) {
+      const redirectUrl = new URL(redirectUri!)
+      redirectUrl.searchParams.set('token', token)
+      redirectUrl.searchParams.set('state', searchParams.get('state') || '')
+
+      return NextResponse.redirect(redirectUrl)
+    }
+
+    // Regular web authentication
+    return NextResponse.redirect(
+      new URL(callbackUrl, 'https://haxa.ai/')
+    )
   } catch (error) {
     console.error('Authentication error:', error)
     return NextResponse.redirect(new URL('/?error=auth_failed', request.url))
```

Key changes:
1. Added detection for VSCode authentication via `redirect_uri`
2. For VSCode auth, construct a redirect URL with:
   - Authentication token
   - Original state parameter
3. Use `uuidv4()` for consistent UUID generation
4. Preserve the original state for VSCode extension's authentication flow

This modification ensures that:
- Regular web authentication continues to work
- VSCode extension receives a token via URI callback
- State parameter is preserved for security validation

References:
- OAuth 2.0 Specification: https://oauth.net/2/
- VSCode Extension URI Handling: https://code.visualstudio.com/api/references/vscode-api#env.asExternalUri

Reasoning:
- The implementation follows OAuth 2.0 best practices
- Maintains separation of concerns between web and VSCode authentication
- Provides a secure token exchange mechanism
- Allows flexible redirect handling


## Alexander's notes (TODO):
If user is already signed in, google auth shouldn't be initiated.

> on the landing website /auth/vscode generates a callback url for authenticating vscode-extension in vscode. It looks like this:
`vscode://haxa-ai.authenticate%3FwindowId%3D8?token=eyJhbGciOiJIUzI1NiJ9.eyJ1c2VySWQiOjEsImVtYWlsIjoiYWxleGFuZGVyLm1hbnp5dWtAaGF4YS5haSIsImlhdCI6MTczOTAxNjgwMCwiZXhwIjoxNzM5MTAzMjAwfQ._DdIDzr9MvjhthT_uN6FDAcBCfYfxhd3X82K6TzzgIA&state=4e644a42-9c4f-44fe-98be-f992d90ec938`
> When user clicks the link, VSCode actvates, which is correct. Although, `console.log('[haxa-ai.handleUri]'` does not seem to be triggered in `extension.ts`: these messages never arrive in the console. What can be the problem?

> On the landing website, /auth/vscode generates such value of callback url: vscode://haxa-ai.authenticate%3FwindowId%3D8?token=eyJhbGciOiJIUzI1NiJ9.eyJ1c2VySWQiOjEsImVtYWlsIjoiYWxleGFuZGVyLm1hbnp5dWtAaGF4YS5haSIsImlhdCI6MTczOTAxNjgwMCwiZXhwIjoxNzM5MTAzMjAwfQ._DdIDzr9MvjhthT_uN6FDAcBCfYfxhd3X82K6TzzgIA&state=4e644a42-9c4f-44fe-98be-f992d90ec938
> VSCode opens, but it doesn't seem to pass the url to my extension: async handleUri(uri: vscode.Uri) { is never triggered in extension.ts.
> Mind vscode-extension/package.json
> What can be wrong in the code?


## contributes.authentication works for vscode extensions?

> how does contributes.authentication works for vscode extensions? Because previous suggestions didn't work.


# Webview menus (top right)
https://claude.ai/chat/74fa63c1-7956-40f3-9b7e-f263a44097c0

```
// First, add the button to package.json
{
  "commands": [
    {
      "command": "haxa-ai.authButton",
      "title": "Auth Status",
      "icon": "$(key)" // default icon
    }
  ],
  "menus": {
    "view/title": [
      <!-- Dropdown / three dots -->
      {
        "command": "haxa-ai.authenticate",
        "title": "Authenticate Haxa.AI",
        "category": "Haxa.AI",
        "icon": "$(key)"
      },
      <!-- Button -->
      {
        "command": "haxa-ai.authenticate",
        "title": "Authenticate Haxa.AI",
        "category": "Haxa.AI",
        "group": "navigation",
        "icon": "$(key)"
      }
    ]
  }
}

// Then in your extension.ts:
let isAuthenticated = false; // track auth state

// Register a command that returns dynamic text/icon
context.subscriptions.push(
    vscode.commands.registerCommand('haxa-ai.authButton', () => {
        return {
            command: 'haxa-ai.authenticate', // actual command to execute when clicked
            title: isAuthenticated ? 'Signed In' : 'Sign In',
            tooltip: isAuthenticated ? 'Click to sign out' : 'Click to authenticate',
            icon: isAuthenticated ? '$(check)' : '$(key)'
        };
    })
);

// When auth state changes, call this to refresh the button:
vscode.commands.executeCommand('setContext', 'haxa-ai.isAuthenticated', isAuthenticated);
``
