"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.HaxaAiAuthenticationProvider = void 0;
const vscode_1 = require("vscode");
class HaxaAiPatSession {
    accessToken;
    // We don't know the user's account name, so we'll just use a constant
    account = { id: HaxaAiAuthenticationProvider.id, label: '[label] Personal Access Token' };
    // This id isn't used for anything in this example, so we set it to a constant
    id = HaxaAiAuthenticationProvider.id;
    // We don't know what scopes the PAT has, so we have an empty array here.
    scopes = [];
    /**
     *
     * @param accessToken The personal access token to use for authentication
     */
    constructor(accessToken) {
        this.accessToken = accessToken;
    }
}
class HaxaAiAuthenticationProvider {
    secretStorage;
    static id = 'haxapat';
    static secretKey = 'HaxaPATSecretKey';
    // this property is used to determine if the token has been changed in another window of VS Code.
    // It is used in the checkForUpdates function.
    currentToken;
    initializedDisposable;
    _onDidChangeSessions = new vscode_1.EventEmitter();
    get onDidChangeSessions() {
        return this._onDidChangeSessions.event;
    }
    constructor(secretStorage) {
        this.secretStorage = secretStorage;
    }
    dispose() {
        this.initializedDisposable?.dispose();
    }
    ensureInitialized() {
        if (this.initializedDisposable === undefined) {
            void this.cacheTokenFromStorage();
            this.initializedDisposable = vscode_1.Disposable.from(
            // This onDidChange event happens when the secret storage changes in _any window_ since
            // secrets are shared across all open windows.
            this.secretStorage.onDidChange(e => {
                if (e.key === HaxaAiAuthenticationProvider.secretKey) {
                    void this.checkForUpdates();
                }
            }), 
            // This fires when the user initiates a "silent" auth flow via the Accounts menu.
            vscode_1.authentication.onDidChangeSessions(e => {
                if (e.provider.id === HaxaAiAuthenticationProvider.id) {
                    void this.checkForUpdates();
                }
            }));
        }
    }
    // This is a crucial function that handles whether or not the token has changed in
    // a different window of VS Code and sends the necessary event if it has.
    async checkForUpdates() {
        const added = [];
        const removed = [];
        const changed = [];
        const previousToken = await this.currentToken;
        const session = (await this.getSessions())[0];
        if (session?.accessToken && !previousToken) {
            added.push(session);
        }
        else if (!session?.accessToken && previousToken) {
            removed.push(session);
        }
        else if (session?.accessToken !== previousToken) {
            changed.push(session);
        }
        else {
            return;
        }
        void this.cacheTokenFromStorage();
        this._onDidChangeSessions.fire({ added: added, removed: removed, changed: changed });
    }
    cacheTokenFromStorage() {
        this.currentToken = this.secretStorage.get(HaxaAiAuthenticationProvider.secretKey);
        return this.currentToken;
    }
    // This function is called first when `vscode.authentication.getSessions` is called.
    async getSessions(_scopes) {
        this.ensureInitialized();
        const token = await this.cacheTokenFromStorage();
        return token ? [new HaxaAiPatSession(token)] : [];
    }
    // This function is called after `this.getSessions` is called and only when:
    // - `this.getSessions` returns nothing but `createIfNone` was set to `true` in `vscode.authentication.getSessions`
    // - `vscode.authentication.getSessions` was called with `forceNewSession: true`
    // - The end user initiates the "silent" auth flow via the Accounts menu
    async createSession(_scopes) {
        this.ensureInitialized();
        // Prompt for the PAT.
        const token = await vscode_1.window.showInputBox({
            ignoreFocusOut: true,
            placeHolder: 'Personal access token',
            prompt: 'Enter an Haxa.AI Personal Access Token (PAT).',
            password: true,
        });
        // Note: this example doesn't do any validation of the token beyond making sure it's not empty.
        if (!token) {
            throw new Error('PAT is required');
        }
        // Don't set `currentToken` here, since we want to fire the proper events in the `checkForUpdates` call
        await this.secretStorage.store(HaxaAiAuthenticationProvider.secretKey, token);
        console.log('[HaxaAiAuthenticationProvider][createSession] Successfully logged in to Haxa.AI');
        return new HaxaAiPatSession(token);
    }
    // This function is called when the end user signs out of the account.
    async removeSession(_sessionId) {
        const token = await this.currentToken;
        if (!token) {
            return;
        }
        await this.secretStorage.delete(HaxaAiAuthenticationProvider.secretKey);
        this._onDidChangeSessions.fire({
            added: [],
            changed: [],
            removed: [new HaxaAiPatSession(token)]
        });
    }
}
exports.HaxaAiAuthenticationProvider = HaxaAiAuthenticationProvider;
//# sourceMappingURL=authProvider.js.map