"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || (function () {
    var ownKeys = function(o) {
        ownKeys = Object.getOwnPropertyNames || function (o) {
            var ar = [];
            for (var k in o) if (Object.prototype.hasOwnProperty.call(o, k)) ar[ar.length] = k;
            return ar;
        };
        return ownKeys(o);
    };
    return function (mod) {
        if (mod && mod.__esModule) return mod;
        var result = {};
        if (mod != null) for (var k = ownKeys(mod), i = 0; i < k.length; i++) if (k[i] !== "default") __createBinding(result, mod, k[i]);
        __setModuleDefault(result, mod);
        return result;
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
exports.GitApply = void 0;
const fs = __importStar(require("fs"));
const path = __importStar(require("path"));
/**
 * A fault-tolerant implementation of git apply that can handle unified diffs
 */
class GitApply {
    workspaceFolder;
    errors = [];
    statistics = {
        totalHunks: 0,
        appliedHunks: 0,
        failedHunks: 0,
        partialMatches: 0,
        filesProcessed: 0,
        filesModified: 0,
        filesFailed: 0
    };
    /**
     * Creates a new GitApply instance
     * @param workspaceFolder The root folder where diffs should be applied
     */
    constructor(workspaceFolder) {
        this.workspaceFolder = workspaceFolder;
    }
    /**
     * Applies a unified diff to files in the workspace
     * @param diff The unified diff to apply
     * @returns ApplyResult containing modified files and error information
     */
    apply(diff) {
        this.errors = [];
        this.resetStatistics();
        const modifiedFiles = [];
        try {
            const files = this.parseFiles(diff);
            for (const [filePath, fileHunks] of Object.entries(files)) {
                this.statistics.filesProcessed++;
                const absolutePath = path.join(this.workspaceFolder, filePath);
                try {
                    // Check if file exists
                    if (!fs.existsSync(absolutePath)) {
                        throw new Error(`File not found: ${filePath}`);
                    }
                    // Read file content
                    const content = fs.readFileSync(absolutePath, 'utf-8');
                    const lines = content.split('\n');
                    // Apply hunks
                    const modifiedContent = this.applyHunks(lines, fileHunks);
                    // Write back if changes were made
                    if (modifiedContent) {
                        fs.writeFileSync(absolutePath, modifiedContent.join('\n'));
                        modifiedFiles.push(filePath);
                        this.statistics.filesModified++;
                    }
                }
                catch (error) {
                    this.statistics.filesFailed++;
                    this.errors.push({
                        type: 'FILE_ERROR',
                        message: error instanceof Error ? error.message : 'Unknown file error',
                        filePath
                    });
                }
            }
            return {
                success: this.errors.length === 0,
                modifiedFiles,
                errors: this.errors,
                statistics: this.statistics
            };
        }
        catch (error) {
            this.errors.push({
                type: 'PARSE_ERROR',
                message: error instanceof Error ? error.message : 'Unknown error during diff application'
            });
            return {
                success: false,
                modifiedFiles: [],
                errors: this.errors,
                statistics: this.statistics
            };
        }
    }
    resetStatistics() {
        this.statistics = {
            totalHunks: 0,
            appliedHunks: 0,
            failedHunks: 0,
            partialMatches: 0,
            filesProcessed: 0,
            filesModified: 0,
            filesFailed: 0
        };
    }
    parseFiles(diff) {
        const files = {};
        const lines = diff.split('\n');
        let currentFile = null;
        let currentHunk = null;
        const hunkHeaderRegex = /^@@ -(\d+)(?:,(\d+))? \+(\d+)(?:,(\d+))? @@/;
        let lineNumber = 0;
        for (const line of lines) {
            lineNumber++;
            try {
                // Parse file header
                if (line.startsWith('diff --git')) {
                    currentFile = null;
                    continue;
                }
                // Parse file paths
                if (line.startsWith('--- a/')) {
                    const newLine = lines[lineNumber]?.startsWith('+++ b/');
                    if (newLine) {
                        currentFile = line.slice(6); // Remove '--- a/' prefix
                        files[currentFile] = [];
                        continue;
                    }
                }
                if (!currentFile)
                    continue;
                // Skip index and +++ lines
                if (line.startsWith('index') || line.startsWith('+++')) {
                    continue;
                }
                // Parse hunk header
                const headerMatch = line.match(hunkHeaderRegex);
                if (headerMatch) {
                    if (currentHunk) {
                        files[currentFile].push(currentHunk);
                    }
                    currentHunk = {
                        originalStart: parseInt(headerMatch[1], 10),
                        originalLines: parseInt(headerMatch[2] || '1', 10),
                        newStart: parseInt(headerMatch[3], 10),
                        newLines: parseInt(headerMatch[4] || '1', 10),
                        content: []
                    };
                    this.statistics.totalHunks++;
                    continue;
                }
                // Add line to current hunk
                if (currentHunk && currentFile && line.length > 0) {
                    currentHunk.content.push(line);
                }
            }
            catch (error) {
                this.errors.push({
                    type: 'PARSE_ERROR',
                    message: `Failed to parse line: ${line}`,
                    lineNumber,
                    filePath: currentFile || undefined
                });
            }
        }
        // Add last hunk if exists
        if (currentHunk && currentFile) {
            files[currentFile].push(currentHunk);
        }
        return files;
    }
    applyHunks(content, hunks) {
        const result = [...content];
        let modified = false;
        // Apply hunks in reverse order to maintain line numbers
        for (const hunk of hunks.reverse()) {
            try {
                if (this.applyHunk(result, hunk)) {
                    modified = true;
                }
                this.statistics.appliedHunks++;
            }
            catch (error) {
                this.statistics.failedHunks++;
                if (error instanceof Error) {
                    const hunkError = {
                        type: 'HUNK_ERROR',
                        message: error.message,
                        hunk: {
                            header: `@@ -${hunk.originalStart},${hunk.originalLines} +${hunk.newStart},${hunk.newLines} @@`,
                            originalStart: hunk.originalStart,
                            originalLines: hunk.originalLines,
                            newStart: hunk.newStart,
                            newLines: hunk.newLines
                        }
                    };
                    this.errors.push(hunkError);
                }
                // Continue with next hunk even if one fails
                continue;
            }
        }
        return modified ? result : null;
    }
    applyHunk(content, hunk) {
        let originalIndex = hunk.originalStart - 1;
        let newContent = [];
        let contextMatches = 0;
        let requiredContextMatches = 0;
        // Count context lines in hunk
        for (const line of hunk.content) {
            if (line.startsWith(' ')) {
                requiredContextMatches++;
            }
        }
        // Verify context with fuzzy matching
        for (const line of hunk.content) {
            if (line.startsWith('-') || line.startsWith(' ')) {
                const expectedLine = line.slice(1);
                const actualLine = content[originalIndex];
                // Use fuzzy matching for context lines
                if (line.startsWith(' ')) {
                    if (this.fuzzyMatch(expectedLine, actualLine)) {
                        contextMatches++;
                    }
                    else {
                        this.errors.push({
                            type: 'CONTEXT_MISMATCH',
                            message: 'Context line mismatch',
                            lineNumber: originalIndex + 1,
                            expectedContent: expectedLine,
                            actualContent: actualLine
                        });
                    }
                }
                originalIndex++;
            }
            if (line.startsWith('+')) {
                newContent.push(line.slice(1));
            }
            else if (line.startsWith(' ')) {
                newContent.push(line.slice(1));
            }
        }
        // Allow the hunk to be applied if most context lines match
        const contextMatchRatio = contextMatches / requiredContextMatches;
        if (contextMatchRatio >= 0.8) {
            if (contextMatchRatio < 1.0) {
                this.statistics.partialMatches++;
            }
            // Apply the changes
            content.splice(hunk.originalStart - 1, hunk.originalLines, ...newContent);
            return true;
        }
        else {
            throw new Error(`Insufficient context matches: ${(contextMatchRatio * 100).toFixed(1)}%`);
        }
    }
    fuzzyMatch(expected, actual) {
        if (!expected || !actual)
            return false;
        // Remove whitespace differences
        expected = expected.trim();
        actual = actual.trim();
        if (expected === actual)
            return true;
        // Calculate Levenshtein distance
        const distance = this.levenshteinDistance(expected, actual);
        const maxLength = Math.max(expected.length, actual.length);
        // Allow up to 20% difference
        return distance <= maxLength * 0.2;
    }
    levenshteinDistance(a, b) {
        const matrix = [];
        // Initialize matrix
        for (let i = 0; i <= b.length; i++) {
            matrix[i] = [i];
        }
        for (let j = 0; j <= a.length; j++) {
            matrix[0][j] = j;
        }
        // Fill matrix
        for (let i = 1; i <= b.length; i++) {
            for (let j = 1; j <= a.length; j++) {
                if (b.charAt(i - 1) === a.charAt(j - 1)) {
                    matrix[i][j] = matrix[i - 1][j - 1];
                }
                else {
                    matrix[i][j] = Math.min(matrix[i - 1][j - 1] + 1, // substitution
                    Math.min(matrix[i][j - 1] + 1, // insertion
                    matrix[i - 1][j] + 1 // deletion
                    ));
                }
            }
        }
        return matrix[b.length][a.length];
    }
}
exports.GitApply = GitApply;
//# sourceMappingURL=git-apply.js.map