"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
class GitDiffApplier {
    static normalizeLineEndings(text) {
        return text.replace(/\r\n/g, '\n').replace(/\r/g, '\n');
    }
    static parseHunkHeader(header) {
        const match = header.match(/@@ -(\d+)(?:,(\d+))? \+(\d+)(?:,(\d+))? @@/);
        if (!match) {
            throw new Error(`Invalid hunk header: ${header}`);
        }
        return {
            oldStart: parseInt(match[1], 10) || 1,
            oldLines: parseInt(match[2], 10) || 1,
            newStart: parseInt(match[3], 10) || 1,
            newLines: parseInt(match[4], 10) || 1,
        };
    }
    static parseDiff(rawDiff) {
        const files = [];
        const diffLines = this.normalizeLineEndings(rawDiff).split('\n');
        let currentFile = null;
        let currentHunk = null;
        for (const line of diffLines) {
            if (line.startsWith('diff --git')) {
                if (currentFile && currentHunk) {
                    currentFile.hunks.push(currentHunk);
                    files.push(currentFile);
                }
                // Initialize with all required properties
                currentFile = {
                    oldFile: '',
                    newFile: '',
                    hunks: [],
                };
                currentHunk = null;
                continue;
            }
            if (!currentFile)
                continue;
            if (line.startsWith('--- ')) {
                currentFile.oldFile = line.slice(4);
            }
            else if (line.startsWith('+++ ')) {
                currentFile.newFile = line.slice(4);
            }
            else if (line.startsWith('@@ ')) {
                if (currentHunk) {
                    currentFile.hunks.push(currentHunk);
                }
                const header = this.parseHunkHeader(line);
                // Initialize with all required properties
                currentHunk = {
                    ...header,
                    context: [],
                    removed: [],
                    added: [],
                };
            }
            else if (currentHunk) {
                // Skip empty lines in the diff that might appear after the hunk header
                if (!line &&
                    currentHunk.context.length === 0 &&
                    currentHunk.removed.length === 0 &&
                    currentHunk.added.length === 0) {
                    continue;
                }
                if (line.startsWith(' ')) {
                    currentHunk.context.push(line.slice(1));
                }
                else if (line.startsWith('-')) {
                    currentHunk.removed.push(line.slice(1));
                }
                else if (line.startsWith('+')) {
                    currentHunk.added.push(line.slice(1));
                }
            }
        }
        // Don't forget to add the last hunk and file
        if (currentFile && currentHunk) {
            currentFile.hunks.push(currentHunk);
            files.push(currentFile);
        }
        return files;
    }
    static findHunkPosition(fileContent, hunk) {
        if (!fileContent.trim()) {
            return 0;
        }
        const fileLines = fileContent.split('\n');
        // If we have context lines, try to find an exact match first
        if (hunk.context.length > 0 || hunk.removed.length > 0) {
            const searchLines = [...hunk.context.slice(0, 3), ...hunk.removed, ...hunk.context.slice(-3)];
            for (let i = 0; i <= fileLines.length - searchLines.length; i++) {
                let matches = true;
                for (let j = 0; j < searchLines.length; j++) {
                    if (fileLines[i + j] !== searchLines[j]) {
                        matches = false;
                        break;
                    }
                }
                if (matches) {
                    return i;
                }
            }
        }
        // If no exact match found, use line number from hunk header
        return Math.max(0, hunk.oldStart - 1);
    }
    static applyDiff(fileContent, rawDiff) {
        const normalizedContent = this.normalizeLineEndings(fileContent);
        const files = this.parseDiff(rawDiff);
        let result = normalizedContent;
        for (const file of files) {
            for (const hunk of file.hunks) {
                if (!result.trim() && hunk.added.length > 0) {
                    result = hunk.added.join('\n');
                    continue;
                }
                const position = this.findHunkPosition(result, hunk);
                const lines = result.split('\n');
                // Ensure we're not going out of bounds
                if (position > lines.length) {
                    throw new Error(`Invalid hunk position: ${position} exceeds file length ${lines.length}`);
                }
                // Apply the hunk changes
                const before = lines.slice(0, position);
                const after = lines.slice(position + (hunk.removed.length || 0));
                result = [...before, ...hunk.added, ...after].join('\n');
            }
        }
        return result;
    }
}
exports.default = GitDiffApplier;
//# sourceMappingURL=git-diff-applier.js.map