"use strict";
// https://claude.ai/chat/2e83b8fc-574c-4be2-909e-61f181a393be
// TODO: Move to https://github.com/ecomfe/gitdiff-parser
Object.defineProperty(exports, "__esModule", { value: true });
exports.applyDiff = applyDiff;
exports.parseUnifiedDiff = parseUnifiedDiff;
exports.formatDiffResult = formatDiffResult;
const robust_apply_hunk_1 = require("./robust-apply-hunk");
function parseUnifiedDiff(diffText) {
    const files = [];
    let currentFile = null;
    let currentHunk = null;
    const lines = diffText.split('\n');
    let i = 0;
    while (i < lines.length) {
        const line = lines[i];
        // Parse file header
        if (line.startsWith('--- ')) {
            const oldFile = line.slice(4);
            i++;
            if (i < lines.length && lines[i].startsWith('+++ ')) {
                const newFile = lines[i].slice(4);
                currentFile = { oldFile, newFile, hunks: [] };
                files.push(currentFile);
            }
        }
        // Parse hunk header
        else if (line.startsWith('@@ ')) {
            const match = line.match(/@@ -(\d+)(?:,\d+)? \+\d+(?:,\d+)? @@/);
            if (match && currentFile) {
                const startLine = parseInt(match[1], 10) - 1; // Convert to 0-based
                currentHunk = {
                    startLine,
                    contextBefore: [],
                    changes: [],
                    contextAfter: []
                };
                currentFile.hunks.push(currentHunk);
                // The rest of the line after @@ might be context
                const contextStart = line.indexOf('@@', line.indexOf('@@') + 2) + 2;
                if (contextStart < line.length) {
                    const contextLine = line.slice(contextStart).trim();
                    if (contextLine) {
                        currentHunk.contextBefore.push(contextLine);
                    }
                }
            }
        }
        // Parse hunk content
        else if (currentHunk) {
            if (line.startsWith(' ')) {
                // Context line
                if (currentHunk.changes.length === 0) {
                    currentHunk.contextBefore.push(line.slice(1));
                }
                else {
                    currentHunk.contextAfter.push(line.slice(1));
                }
            }
            else if (line.startsWith('-')) {
                currentHunk.contextAfter = []; // Reset context after when new changes start
                currentHunk.changes.push(['-', line.slice(1)]);
            }
            else if (line.startsWith('+')) {
                currentHunk.contextAfter = []; // Reset context after when new changes start
                currentHunk.changes.push(['+', line.slice(1)]);
            }
            else if (line === '') {
                // Empty line could be end of hunk
                currentHunk = null;
            }
        }
        i++;
    }
    return files;
}
function applyDiff(sourceContent, diffText, fuzz = 1) {
    try {
        // Parse the source content into lines
        const sourceLines = sourceContent.split('\n');
        // Parse the diff
        const files = parseUnifiedDiff(diffText);
        if (files.length === 0) {
            return {
                success: false,
                error: 'No valid diff found',
                appliedHunks: 0,
                totalHunks: 0,
                result: sourceLines
            };
        }
        // We'll only handle single file diffs for now
        const fileDiff = files[0];
        let currentLines = sourceLines;
        let appliedHunks = 0;
        // Apply each hunk in sequence
        for (const hunk of fileDiff.hunks) {
            const [newLines, success] = (0, robust_apply_hunk_1.applyHunk)(currentLines, hunk, fuzz);
            if (!success) {
                return {
                    success: false,
                    error: `Failed to apply hunk starting at line ${hunk.startLine + 1}`,
                    appliedHunks,
                    totalHunks: fileDiff.hunks.length,
                    result: currentLines
                };
            }
            console.log('[robustApplyDiff][applyDiff]newLines', newLines);
            currentLines = newLines;
            appliedHunks++;
        }
        return {
            success: true,
            appliedHunks,
            totalHunks: fileDiff.hunks.length,
            result: currentLines
        };
    }
    catch (error) {
        return {
            success: false,
            error: error instanceof Error ? error.message : 'Unknown error',
            appliedHunks: 0,
            totalHunks: 0,
            result: sourceContent.split('\n')
        };
    }
}
// Helper function to get a nice string output
function formatDiffResult(result) {
    if (!result.success) {
        return `Failed to apply diff: ${result.error}\n` +
            `Applied ${result.appliedHunks}/${result.totalHunks} hunks\n\n` +
            result.result.join('\n');
    }
    return `Successfully applied ${result.appliedHunks} hunks\n\n` +
        result.result.join('\n');
}
//# sourceMappingURL=robust-apply-diff.js.map