"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.applyHunk = applyHunk;
const SIMILARITY_THRESHOLD = 0.7;
const EPSILON = 1e-10;
function normalizeLineEndings(lines) {
    return lines.map(line => line.replace(/\r\n?/g, '\n'));
}
function normalizeLine(line) {
    const trimmed = line.trim();
    return {
        raw: line,
        trimmed,
        words: new Set(trimmed.split(/\s+/).filter(Boolean))
    };
}
function inferIndentation(contextLines) {
    for (const line of contextLines) {
        const match = line.match(/^(\s+)/);
        if (match)
            return match[1];
    }
    return '';
}
function calculateSimilarity(a, b) {
    // Fast path for exact matches
    if (a.trimmed === b.trimmed)
        return 1.0;
    if (a.trimmed === '' && b.trimmed === '')
        return 1.0;
    if (a.trimmed === '' || b.trimmed === '')
        return 0.0;
    const commonWords = new Set([...a.words].filter(word => b.words.has(word)));
    return commonWords.size / Math.max(a.words.size, b.words.size);
}
function findBestMatch(needleLines, haystackLines, startIdx, endIdx, preferredPosition = null) {
    const normalizedNeedle = needleLines.map(normalizeLine);
    const normalizedHaystack = haystackLines.slice(startIdx, endIdx).map(normalizeLine);
    // Guard against insufficient haystack length
    if (normalizedHaystack.length < normalizedNeedle.length) {
        return { position: -1, score: -1 };
    }
    let bestScore = -1;
    let bestPos = -1;
    // Try each possible starting position
    for (let i = 0; i <= normalizedHaystack.length - normalizedNeedle.length; i++) {
        let score = 0;
        for (let j = 0; j < normalizedNeedle.length; j++) {
            if (i + j < normalizedHaystack.length) {
                const similarity = calculateSimilarity(normalizedNeedle[j], normalizedHaystack[i + j]);
                // Weight exact matches higher
                score += similarity === 1.0 ? 3 : similarity;
            }
        }
        // Boost score if position matches preferred position
        const currentPos = startIdx + i;
        if (preferredPosition !== null && currentPos === preferredPosition) {
            score += 0.5; // Small boost for matching the expected position
        }
        if (score > bestScore) {
            bestScore = score;
            bestPos = currentPos;
        }
    }
    return { position: bestPos, score: bestScore };
}
function findStartPosition(sourceLines, hunk, fuzz) {
    const searchStart = Math.max(0, hunk.startLine - fuzz);
    const searchEnd = Math.min(sourceLines.length, hunk.startLine + (hunk.contextBefore.length || 1) + fuzz);
    // If we have context before, use it to find the best position
    if (hunk.contextBefore.length > 0) {
        const { position, score } = findBestMatch(hunk.contextBefore, sourceLines, searchStart, searchEnd, hunk.startLine);
        return score >= 0 ? position : null;
    }
    // If we have no context before, try to match the first removal line
    const firstRemoval = hunk.changes.find(([op]) => op === '-');
    if (firstRemoval) {
        const { position, score } = findBestMatch([firstRemoval[1]], sourceLines, searchStart, searchEnd, hunk.startLine);
        return score >= SIMILARITY_THRESHOLD ? position : null;
    }
    // If we only have additions and no context, use the start line if it's in bounds
    if (hunk.startLine >= 0 && hunk.startLine <= sourceLines.length) {
        return hunk.startLine;
    }
    return null;
}
function applyHunk(sourceLines, hunk, fuzz = 3) {
    // Normalize all input
    sourceLines = normalizeLineEndings(sourceLines);
    hunk.contextBefore = normalizeLineEndings(hunk.contextBefore);
    hunk.contextAfter = normalizeLineEndings(hunk.contextAfter);
    hunk.changes = hunk.changes.map(([op, line]) => [op, line.replace(/\r\n?/g, '\n')]);
    // Make working copy of source
    const result = [...sourceLines];
    // Find the actual start position
    const actualStart = findStartPosition(sourceLines, hunk, fuzz);
    if (actualStart === null) {
        return [sourceLines, false];
    }
    // Apply changes
    let currentLine = actualStart;
    const newLines = [];
    const indentation = inferIndentation(hunk.contextBefore);
    for (const [op, line] of hunk.changes) {
        if (op === '-') {
            // Verify line being removed matches (with some fuzziness)
            if (currentLine >= result.length) {
                return [sourceLines, false];
            }
            const normalizedSource = normalizeLine(result[currentLine]);
            const normalizedRemove = normalizeLine(line);
            // Fast path for exact matches
            if (normalizedSource.trimmed === normalizedRemove.trimmed) {
                currentLine++;
                continue;
            }
            // Try partial matching
            const similarity = calculateSimilarity(normalizedSource, normalizedRemove);
            if (similarity + EPSILON <= SIMILARITY_THRESHOLD) {
                return [sourceLines, false];
            }
            currentLine++;
        }
        else if (op === '+') {
            // Preserve indentation for new lines
            newLines.push(line.startsWith(' ') ? line : indentation + line);
        }
    }
    // Replace the affected lines
    result.splice(actualStart, currentLine - actualStart, ...newLines);
    // Verify context after if it exists
    if (hunk.contextAfter.length > 0) {
        // Check if we have enough lines remaining
        if (currentLine + hunk.contextAfter.length > result.length) {
            return [sourceLines, false];
        }
        const expectedAfter = hunk.contextAfter.map(normalizeLine);
        const actualAfter = result
            .slice(currentLine, currentLine + hunk.contextAfter.length)
            .map(normalizeLine);
        const matchingLines = expectedAfter.reduce((count, line, i) => {
            const similarity = calculateSimilarity(line, actualAfter[i]);
            return count + (similarity + EPSILON > SIMILARITY_THRESHOLD ? 1 : 0);
        }, 0);
        if (matchingLines < expectedAfter.length * SIMILARITY_THRESHOLD) {
            return [sourceLines, false];
        }
    }
    return [result, true];
}
//# sourceMappingURL=robust-apply-hunk.js.map